
/**
 * All problem classes must implement this interface.
 *
 * @author Dietrich Zerr
 */
public interface Problem {

    /**
     * Returns the whole content of a PDDL domain file. This will be called only
     * once, independent of how many examples of a problem are being generated.
     *
     * @param properties the properties of the problem
     * @return a PDDL domain description
     */
    public String getPDDLDomain(int[] properties) throws IllegalArgumentException;

    /**
     * Returns the whole content of a PDDL problem file. The problem shoud be
     * newly generated every time this function is called, not in the
     * constructor. If the generator has to generate multiple examples of a
     * problems at once, your class will still be created only once and this
     * method will then be called multiple times with the respective 
     * instance number as instanceNumber (-> watch out for default properties
     * that are not given every time).
     *
     * @param properties the properties of the problem
     * @param instanceNumber the number of the problem instance
     * @return a PDDL problem description
     */
    public String getPDDLProblem(int[] properties, int instanceNumber) throws IllegalArgumentException;

    /**
     * Returns the Name that should be used for the generated problem PDDL
     * file(s). The files will look like this: < br >
     * [getProblemFileName][problem-file-suffix][n][pddl-file-ending] < br >
     * e.g. sokoban-problem1.pddl sokoban-problem2.pddl ...< br >
     * If [getProblemFileName] is null, the name of the problem javafile will be
     * used. [n] is the number (importent if you create several instances in one
     * go). [problem-file-suffix] and [pddl-file-ending] are specified in the
     * properties file. Please beware of the naming restrictions of your system.
     *
     * @return the problem-name to be used to name the output files or null
     */
    public String getProblemFileName();

    /**
     * Returns the Name that should be used for the generated domain PDDL file.
     * The file will look like this: < br >
     * [getDomainFileName][problem-file-suffix][pddl-file-ending] < br >
     * e.g. sokoban-domain1.pddl sokoban-domain2.pddl ...< br >
     * If [getDomainFileName] is null, the name of the problem javafile will be
     * used. [domain-file-suffix] and [pddl-file-ending] are specified in the
     * properties file. Please beware of the naming restrictions of your system.
     *
     * @return the problem-name to be used to name the output files or null
     */
    public String getDomainFileName();
}
